passwd = [
# commented out by default - use /usr/share/pyftpd/conf_auth_db.py to change/add users
#('test', 'test', 'CY9rzUYh03PK4k6DJie09g=='),
# ('user', 'users', '7hHLsZBS6AsHqsDKBgwj7g=='),
# ('roxon', 'users', 'ItZ2pB7rPmzFV6hrtdnZ7A==')
]


